-- RevengeofTheDead 1.2 by GraySloth 2014-May-20

----Config-----
local special_ghost_percent = 2/3 --the chance of a ghost being special during a revenge attack, set to 0 for no special ghost, ghosts by any other means will still always be normal.
TUNING.special_ghost_drop_gem_chance = 1 --1 means 100% chance 1/2 would mean 50%, and 0.5 would also mean 50%
local specialOnlyGems = true --if true special ghost will not drop trinkes or other loots added to normal ghosts.

local trinket_chance = 1/3 --the chance for normal ghost dug from graves and during revenge attacks to drops a random trinket, 0 will disable this option. Also special ghost if specialOnlyGems is set to false

local all_ghost_extra_loot = {  --items in this table will be dropped by normal ghost dug from graves and during revenge attacks, also special ghost if specialOnlyGems is set to false.
--this one below is just to give an example of what a normal item would look like.  Just add your own without the double dashes.
--{item = "nightmarefuel", chance = 1/3}, 

}

--disable a perticular colored ghost by commenting out that color below with a double dash like this --"red",
local colors = { 
	"red",
	"blue",
	"purple",
	"orange",
	"yellow",
	"green",
}
------End Config--------

PrefabFiles = {
	"revenge_ghosts",
}

local function giveLoot(ghost)
	if not ghost.components.lootdropper then
		ghost:AddComponent("lootdropper")
	end
	ghost.components.lootdropper:AddChanceLoot("trinket_"..tostring(math.random(GLOBAL.NUM_TRINKETS)), trinket_chance)
	for k,v in pairs(all_ghost_extra_loot) do
		ghost.components.lootdropper:AddChanceLoot(v.item, v.chance)
	end
end

local function onfinishcallback(inst, worker)

    inst.AnimState:PlayAnimation("dug")
    inst:RemoveComponent("workable")
    if GLOBAL.IsDLCEnabled(GLOBAL.REIGN_OF_GIANTS) then
    	inst.components.hole.canbury = true
    end

	if worker then
		if worker.components.sanity then
			worker.components.sanity:DoDelta(-TUNING.SANITY_SMALL)
		end		
		--MOD--
		if worker.components.graysloth_haunted and worker.components.graysloth_haunted.nextevent == 0 then
			worker.components.graysloth_haunted:PlanNextEvent()
		end
		--/MOD--
		if math.random() < .1 then
			local ghost = GLOBAL.SpawnPrefab("ghost")
			local pos = Point(inst.Transform:GetWorldPosition())
			pos.x = pos.x -.3
			pos.z = pos.z -.3
			if ghost then
				--MOD--
				giveLoot(ghost)
				--/MOD--
				ghost.Transform:SetPosition(pos.x, pos.y, pos.z)
			end
		elseif worker.components.inventory then
			local item = nil
			if math.random() < .5 then
				local loots = 
				{
					nightmarefuel = 1,
					amulet = 1,
					gears = 1,
					redgem = 5,
					bluegem = 5,
				}
				item = GLOBAL.weighted_random_choice(loots)
			else
				item = "trinket_"..tostring(math.random(GLOBAL.NUM_TRINKETS))
			end

			if not inst.components.lootdropper then --This is a compatibility fix for mods that try to alter graves
				inst:AddComponent("lootdropper")
			end
			if item then
				inst.components.lootdropper:SpawnLootPrefab(item)
			end
		end
	end	
end

local function MakePoof(ghost)
	ghost.Physics:Stop()
	ghost.AnimState:PlayAnimation("dissipate")
	GLOBAL.SpawnPrefab("collapse_small").Transform:SetPosition(ghost.Transform:GetWorldPosition()) --the titular poof
	ghost:DoTaskInTime(.8,  function() --needs to let the animations play before deleting
		ghost:Remove()
	end)
end

local function onsave(inst, data)
	if inst.removeOnLoad then
		data.removeOnLoad = true
	end
end

local function onload(inst, data)
	if data and data.removeOnLoad then
		if not GLOBAL.GetClock():IsNight() then
			MakePoof(inst)
			return
		end
    	if not specialOnlyGems or inst.prefab == "ghost" then
    		giveLoot(inst)
    	end

    	if inst.prefab ~= "green_revenge_ghost" and inst.prefab ~= "ghost" then
			inst.components.combat:SuggestTarget(GLOBAL.GetPlayer())
			inst:AddTag("epic")
		end
    	inst.removeOnLoad = true
    	inst:ListenForEvent("daytime", function(global, data)
			MakePoof(inst)
		end, GLOBAL.GetWorld())
		inst:DoTaskInTime(TUNING.TOTAL_DAY_TIME * (1 - GLOBAL.GetClock():GetNormTime()), function() 
			MakePoof(inst)
		end)	
	end
end

local function postinitremoveonload(inst)  
	inst.OnSave = onsave
	inst.OnLoad = onload
end
AddPrefabPostInit("ghost", postinitremoveonload)

for i=1,#colors do
	AddPrefabPostInit(colors[i].."_revenge_ghost", postinitremoveonload)
end


local function Spawnghost()
	local ghost = false
	local pos = Point(GLOBAL.GetPlayer().Transform:GetWorldPosition())
    local theta = math.random() * 2 * GLOBAL.PI
    local radius = 6	
	
	if math.random() < special_ghost_percent then
		ghost = GLOBAL.SpawnPrefab(colors[math.random(#colors)].."_revenge_ghost")
		if ghost.prefab ~= "green_revenge_ghost" then
			ghost.components.combat:SuggestTarget(GLOBAL.GetPlayer())
			ghost:AddTag("epic")
		end
	else
		ghost = GLOBAL.SpawnPrefab("ghost")
	end

	if not specialOnlyGems or ghost.prefab == "ghost" then
		giveLoot(ghost)
	end

	if ghost then
		local offset = GLOBAL.FindWalkableOffset(pos, theta, radius, 12, true)
		local spawn_pt = pos+offset
		ghost.removeOnLoad = true
		ghost.Transform:SetPosition(spawn_pt:Get())
		ghost:ListenForEvent("daytime", function(global, data)
			MakePoof(ghost)
		end, GLOBAL.GetWorld())
		ghost:DoTaskInTime(TUNING.TOTAL_DAY_TIME * (1 - GLOBAL.GetClock():GetNormTime()), function() 
			MakePoof(ghost)
		end)	
	end
end

local function postinitmound(inst)  
	local getclock = GLOBAL.GetClock()
	local player = GLOBAL.GetPlayer()
	inst.components.workable:SetOnFinishCallback(onfinishcallback)

	--this is for worlds that already have dug graves when this mod is added.
	inst:DoTaskInTime(0, function() 
		if not inst.components.workable and player.components.graysloth_haunted.nextevent == 0 then
			player.components.graysloth_haunted:PlanNextEvent()
		end
	end)

	inst:ListenForEvent("nighttime", function(global, data)
		inst:DoTaskInTime(0.1, function() --this does two important things, it lets graysloth_haunted.event  get set first and if you sleep it will mean getclock:IsNight() is false because it will be day at that point.
			if not inst.components.workable and getclock:IsNight() and player.components.graysloth_haunted.event then

				if not player:HasTag("insomniac") then
					player:AddTag("insomniac")
					player:DoTaskInTime(TUNING.TOTAL_DAY_TIME * (1 - GLOBAL.GetClock():GetNormTime()), function()
						player:RemoveTag("insomniac")
					end)
				end
				
				local pos = GLOBAL.Vector3(inst.Transform:GetWorldPosition())
				GLOBAL.SpawnPrefab("lightning").Transform:SetPosition(pos:Get())
				GLOBAL.SpawnPrefab("collapse_small").Transform:SetPosition(pos:Get())
				--this undigs the graves, the lightning serves to scary as well as obscure the resetting of the animation.
				inst:AddComponent("workable")
				inst.components.workable:SetWorkAction(GLOBAL.ACTIONS.DIG)
				inst.components.workable:SetWorkLeft(1)
			    if GLOBAL.IsDLCEnabled(GLOBAL.REIGN_OF_GIANTS) then
			    	inst.components.hole.canbury = false
			    end
				inst.AnimState:PlayAnimation("gravedirt")
				inst.components.workable:SetOnFinishCallback(onfinishcallback)

				player.components.graysloth_haunted.nextevent = 0
				--this calls the ghosts.
				inst:DoTaskInTime(1+math.random()*9, function()
					if getclock:IsNight() then 	
						Spawnghost()
					end
				end)		
			end
		end)
	end, GLOBAL.GetWorld())
end

AddPrefabPostInit("mound", postinitmound) 

--this is the unnecessarily hard way to add components that need to load stuff.
AddPrefabPostInit("world", function(inst)
	GLOBAL.assert( GLOBAL.GetPlayer() == nil )
	local player_prefab = GLOBAL.SaveGameIndex:GetSlotCharacter()

	GLOBAL.TheSim:LoadPrefabs( {player_prefab} )
	local oldfn = GLOBAL.Prefabs[player_prefab].fn
	GLOBAL.Prefabs[player_prefab].fn = function()
		local inst = oldfn()

		inst:AddComponent("graysloth_haunted")

		return inst
	end
end)