local assets =
{
	Asset("ANIM", "anim/ghost.zip"),
	Asset("ANIM", "anim/ghost_build.zip"),
	Asset("SOUND", "sound/ghost.fsb"),
}

local prefabs = 
{
    "purplegem",
    "bluegem",
    "redgem",
    "orangegem",
    "yellowgem",
    "greengem",
}

function titlecase(str)
    local result=''
    for word in string.gfind(str, "%S+") do          
        local first = string.sub(word,1,1)
        result = (result .. string.upper(first) ..
            string.lower(string.sub(word,2)) .. ' ')
    end    
    return result
end

local function commonfn(color)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
    inst.entity:AddLight()
    inst.entity:AddPhysics()
    anim:SetBloomEffectHandle( "shaders/anim.ksh" )
    
    MakeGhostPhysics(inst, 1, .5)
    
    inst.Light:SetIntensity(.6)
    inst.Light:SetRadius(.5)
    inst.Light:SetFalloff(.6)
    inst.Light:Enable(true)
    inst.Light:SetColour(180/255, 195/255, 225/255)
    
    local brain = require "brains/ghostbrain"
    inst:SetBrain(brain)
    
    anim:SetBank("ghost")
    anim:SetBuild("ghost_build")
    anim:PlayAnimation("idle", true)
    --inst.AnimState:SetMultColour(1,1,1,.6)

    
    inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("ghost")
    inst:AddTag("noauradamage")

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = TUNING.GHOST_SPEED
    inst.components.locomotor.runspeed = TUNING.GHOST_SPEED
    inst.components.locomotor.directdrive = true
    
    inst:SetStateGraph("SGrevenge_ghost")
    
    
	inst:AddComponent("sanityaura")
    inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED
    
    inst:AddComponent("inspectable")
    
	inst:AddComponent("health")
	inst.components.health:SetMaxHealth(TUNING.GHOST_HEALTH)
    
    inst:AddComponent("combat")
    inst.components.combat.defaultdamage = TUNING.GHOST_DAMAGE
    inst.components.combat.playerdamagepercent = TUNING.GHOST_DMG_PLAYER_PERCENT

    inst:AddComponent("aura")
    inst.components.aura.radius = TUNING.GHOST_RADIUS
    inst.components.aura.tickperiod = TUNING.GHOST_DMG_PERIOD

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:AddChanceLoot(color.."gem", TUNING.special_ghost_drop_gem_chance)

    inst:AddComponent("named")
    inst.components.named:SetName(titlecase(color).."Vengeful Ghost")
    
    ------------------    
    inst.SoundEmitter:PlaySound("dontstarve/ghost/ghost_howl_LP", "howl")
    
    return inst
end

local function red()
    local inst = commonfn("red")
        inst.freezefn = function(attacked, data)
            if data.attacker.components.temperature then
                data.attacker.components.temperature:SetTemperature(data.attacker.components.temperature.current+5)
            end
        end
        inst:ListenForEvent("attacked", inst.freezefn, inst)
        inst.components.health:SetMaxHealth(TUNING.GHOST_HEALTH*2)
        inst.components.health.currenthealth = TUNING.GHOST_HEALTH*2
        inst.Light:SetColour(255/255, 0/255, 0/255)
    return inst
end

local function blue()
    local inst = commonfn("blue")
        inst.freezefn = function(attacked, data)
            --this will freeze creatures that attack it just like the blue amulet.
            if data.attacker.components.freezable then
                data.attacker.components.freezable:AddColdness(0.67)
                data.attacker.components.freezable:SpawnShatterFX()
            elseif data.attacker.components.temperature then
                data.attacker.components.temperature:SetTemperature(data.attacker.components.temperature.current-10)
            end
        end
        inst:ListenForEvent("attacked", inst.freezefn, inst)
        inst.Light:SetColour(69/255, 85/255, 255/255)
    return inst
end

local function purple()
    local inst = commonfn("purple")
        inst.components.sanityaura.aura = -200/60 --minus 200 per minute which is half that of Deerclops.
        inst.Light:SetColour(255/255, 0/255, 255/255)
    return inst
end

local function green()
    local inst = commonfn("green")
        --these next three lines are just here so that it does not confuse the new brain it will get, but do not serve a real purpose.
        inst:AddComponent("knownlocations")
        inst:AddComponent("eater")
        inst.components.eater.foodprefs = {}
        inst.components.locomotor.walkspeed = TUNING.GHOST_SPEED*2
        inst.components.locomotor.runspeed = TUNING.GHOST_SPEED*2
        --this makes sure that the ghost has the right ratio of health to look sad but not be a one hit wonder, with 99 health it still takes some work to kill.
        inst.components.health:SetMaxHealth(TUNING.GHOST_HEALTH*2)
        inst.components.health.currenthealth = (TUNING.GHOST_HEALTH/2-1)
        local rabbitbrain = require "brains/rabbitbrain" --A ghost rabbit! how cute /)^3^(\
        inst:SetBrain(rabbitbrain)
        inst.Light:SetColour(0/255, 220/255, 0/255)
        inst:AddTag("girl")
    return inst
end

local function orange()
    local inst = commonfn("orange")
        inst.randomteleportfn = function(attacked, data)
            if attacked.components.health:IsDead() or not data.attacker or math.random() > .5 then
                return
            end

            local pt = Vector3(data.attacker.Transform:GetWorldPosition())
            
            local spawn_pt = false
            local theta = math.random() * 2 * PI
            local radius = math.random(5,15)
            
            local offset = FindWalkableOffset(pt, theta, radius, 12, true)
            if offset then
                spawn_pt = pt+offset
            end
            
            if spawn_pt then
                SpawnPrefab("small_puff").Transform:SetPosition(pt:Get())
                data.attacker.SoundEmitter:PlaySound("dontstarve/common/staff_blink")
                data.attacker:Hide()
                if data.attacker.components.health then
                    data.attacker.components.health:SetInvincible(true)
                end
                data.attacker:DoTaskInTime(0.25, function() 
                    if data.attacker.components.health then
                        data.attacker.components.health:SetInvincible(false)
                    end
                    data.attacker.Transform:SetPosition(spawn_pt:Get())
                    SpawnPrefab("small_puff").Transform:SetPosition(spawn_pt:Get())
                    data.attacker:Show()
                    data.attacker.SoundEmitter:PlaySound("dontstarve/common/staff_blink")
                    data.attacker.components.combat:SuggestTarget(attacked)
                end)
            end
        end
        inst.components.locomotor.walkspeed = TUNING.GHOST_SPEED*1.25
        inst.components.locomotor.runspeed = TUNING.GHOST_SPEED*1.25
        inst:ListenForEvent("attacked", inst.randomteleportfn, inst)
        inst.Light:SetColour(255/255, 128/255, 0/255)
    return inst
end

local function yellow()
    local inst = commonfn("yellow")
        --inst.Light:SetIntensity(Lerp(0.4, 0.6, 1))
        inst.Light:SetRadius(Lerp(3, 5, 1))
        inst.Light:SetFalloff(.9)
        inst.Light:SetColour(255/255, 225/255, 0/255)
    return inst
end

STRINGS.CHARACTERS.GENERIC.DESCRIBE.RED_REVENGE_GHOST = "What is that, heat? Is that a reverse ghost?" 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.BLUE_REVENGE_GHOST = "It is going to chill me to the bone!" 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.PURPLE_REVENGE_GHOST = "It is driving me mad!" 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.ORANGE_REVENGE_GHOST = "First I am here, then I am there." 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.GREEN_REVENGE_GHOST = "Aw, look how scared it is." 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.YELLOW_REVENGE_GHOST = "It is so bright!" 

return
Prefab( "common/monsters/red_revenge_ghost", red, assets, prefabs ),
Prefab( "common/monsters/blue_revenge_ghost", blue, assets, prefabs ), 
Prefab( "common/monsters/purple_revenge_ghost", purple, assets, prefabs ), 
Prefab( "common/monsters/orange_revenge_ghost", orange, assets, prefabs ),
Prefab( "common/monsters/green_revenge_ghost", green, assets, prefabs ),
Prefab( "common/monsters/yellow_revenge_ghost", yellow, assets, prefabs )